'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _babelCore = require('babel-core');

var babel = _interopRequireWildcard(_babelCore);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var BabelBuilder = function () {
	function BabelBuilder(src, dest, Compiler) {
		_classCallCheck(this, BabelBuilder);

		Compiler.Logger.attach(this, 'BabelBuilder');

		this.log('Initializing');

		this.src = src;
		this.dest = dest;

		this.compiler = Compiler;
		this.getFolders = Compiler.getFolders.bind(Compiler);
		this.buildContext = Compiler.Config.BuildContext;

		this.build(src, dest);

		this.log('Complete');
		Compiler.dispatcher.resolve('BabelBuilder', true);
	}

	_createClass(BabelBuilder, [{
		key: 'build',
		value: function build(src, dest) {
			var _this = this;

			this.log('Transpiling');

			var Compiler = this.compiler;

			var folders = this.getFolders(src).filter(function (dir) {
				return dir.indexOf('node_modules') < 0;
			}).map(function (dir) {
				_this.build(_path2.default.resolve(_path2.default.join(src, dir)), _path2.default.resolve(_path2.default.join(dest, dir)));
			});

			var dirsMade = _fsExtra2.default.mkdirpSync(dest);

			return this.processFiles(src, dest);
		}
	}, {
		key: 'processFiles',
		value: function processFiles(src, dest) {
			var _this2 = this;

			this.log('Processing files');
			return this.getFiles(src).map(function (file) {
				return _this2.writeFile(file, src, dest);
			});
		}
	}, {
		key: 'writeFile',
		value: function writeFile(file, src, dest) {
			var Compiler = this.compiler;

			var _transformFile = this.transformFile(file, src),
			    code = _transformFile.code,
			    err = _transformFile.err;

			if (err) Compiler.compileError(err);

			this.log('Writing file: ' + (dest + file), this.LOG_DEBUG);

			return _fsExtra2.default.writeFileSync(this.renameJsx(_path2.default.resolve(_path2.default.join(dest, file))), code);
		}
	}, {
		key: 'transformFile',
		value: function transformFile(file, src) {
			var _this3 = this;

			this.log('Transforming file: ' + src + file, this.LOG_DEBUG);

			var projectRoot = _path2.default.resolve(this.src);
			var moduleRoot = _path2.default.join(_path2.default.resolve(this.buildContext), './node_modules');

			var ret = { code: null, err: null };
			try {
				ret.code = babel.transformFileSync(_path2.default.resolve(_path2.default.join(src, file)), {
					filename: file,
					presets: ['react', 'latest'],
					plugins: ['transform-es2015-destructuring', 'transform-object-rest-spread'],
					moduleRoot: moduleRoot,
					resolveModuleSource: function resolveModuleSource(source, filename) {
						if (source.indexOf('/') > -1 || source.indexOf('\\') > -1) {
							var resolved = _path2.default.resolve(_path2.default.dirname(filename), source);

							if (resolved.indexOf(projectRoot) > -1) return _this3.renameJsx(source);else return source;
						}

						return source;
					}
				}).code;
			} catch (e) {
				ret.err = e;
			} finally {
				return ret;
			}
		}
	}, {
		key: 'renameJsx',
		value: function renameJsx(filename) {
			var parts = filename.split(/\./g);
			if (parts[parts.length - 1] === 'jsx') {
				parts.pop();
				return parts.concat('js').join('.');
			} else return filename;
		}
	}, {
		key: 'getFiles',
		value: function getFiles(dir) {
			this.log("Retrieving file listing for `" + dir + "`", this.LOG_DEBUG);
			return _fsExtra2.default.readdirSync(dir).filter(function (file) {
				return _fsExtra2.default.statSync(_path2.default.join(dir, file)).isFile() && (file.split(/\./g).pop() === "jsx" || file.split(/\./g).pop() === "js");
			});
		}
	}, {
		key: 'getFolders',
		value: function getFolders(dir) {
			this.log("Retrieving directory listing for `" + dir + "`", this.LOG_DEBUG);
			return _fsExtra2.default.readdirSync(dir).filter(function (file) {
				return _fsExtra2.default.statSync(_path2.default.join(dir, file)).isDirectory();
			});
		}
	}]);

	return BabelBuilder;
}();

exports.default = BabelBuilder;